#!/usr/bin/env python3
"""
This is the code for the Continuous Causal States method
described in the papers:

  Discovering Causal Structure with Reproducing-Kernel Hilbert Space ε-Machines
  by Nicolas Brodu and James P. Crutchfield

  Inferring Kernel ε-Machines: Discovering Structure in Complex Systems
  by Nicolas Brodu and Alexandra M. Jurgens1

  and possibly more to come
  
This code is free software, according to the MIT license
Author & Contact: nicolas.brodu@inria.fr
"""

import matplotlib.pyplot as plt
import matplotlib.ticker
import numpy as np

# Common code shared by the examples for the eigenvalue plots in the article
def plot_eigenvalues(eigenvalues, title, saveprefix=None):
    fig, ax1 = plt.subplots(layout='constrained')
    plt.bar(np.arange(1,9), eigenvalues[1:9]/eigenvalues[1]*100)
    ax2 = ax1.twinx()
    ax1.set_ylim([0,100])
    ax2.ticklabel_format(axis='y', style='sci', scilimits=(0,0))
    ax2.set_ylim([0,eigenvalues[1]])
    ax1.minorticks_on()
    ax2.minorticks_on()
    ax1.xaxis.set_tick_params(which='minor', bottom=False)
    ax1.tick_params(axis='y',labelsize=14)
    ax2.tick_params(axis='y',labelsize=14)
    ax1.tick_params(axis='x',labelsize=14)
    ax1.yaxis.set_minor_locator(matplotlib.ticker.AutoMinorLocator(2))
    ax2.yaxis.set_minor_locator(matplotlib.ticker.AutoMinorLocator(2))
    plt.xticks(np.arange(1,9))
    ax2.set_ylabel("$\\lambda$ value", fontsize=16, labelpad=6)
    ax1.set_ylabel("Percent of $\\lambda_1$", fontsize=16, labelpad=-6)
    ax1.set_xlabel("Eigenvalue index", fontsize=16, labelpad=4)
    plt.title(title, fontsize=16)
    if saveprefix is not None:
        plt.savefig(saveprefix+'.svg')
        plt.savefig(saveprefix+'.pdf')

#Usage:
#plot_eigenvalues(eigenvalues,'Butane molecule example', 'butane_eigenvalues')
