#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Example code for the paper :
  Discovering Causal Structure with Reproducing-Kernel Hilbert Space ε-Machines
  by Nicolas Brodu and James P. Crutchfield

This code is free software, according to the MIT license
Author & Contact: nicolas.brodu@inria.fr
"""

import numpy as np
import matplotlib.pyplot as plt
import scipy
import scipy.linalg as lin
from collections import defaultdict
from numpy.random import default_rng

# This is for running the script from the source code of the uninstalled package
import os
import sys
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
import concaust


import argparse

parser = argparse.ArgumentParser(description='Perform an experiment with the Even process.')
parser.add_argument('--seed', default=42, type=int, help='run exp with that seed')
parser.add_argument('--npast', default=10, type=int, help='past series size')
parser.add_argument('--nfuture', default=10, type=int, help='future series size')
parser.add_argument('--bandwidth', default=0.1, type=float, help='bandwidth (0=auto)')
parser.add_argument('--N', default=10000, type=int, help='Number of training samples')

args = parser.parse_args()

print("=== Even Process experiment with seed = %d ====" % args.seed)
print(args)

window_size = args.npast + args.nfuture

rng = np.random.default_rng(args.seed)
    
series_length = args.N + window_size-1

state = rng.integers(2)
    
series = np.zeros((series_length,))

print("generating the series")
# The even process
for i in range(series_length):
    # State 1 : goes back to state 0
    if state==1:
        series[i] = 1
        state = 0
    # State 0 : half chance to go to either state
    else:
        series[i] = rng.integers(2)
        state = series[i]

print("generating gram matrices")
Gx, Gy, index_map = concaust.series_Gxy(series, args.bandwidth, args.npast, args.nfuture)

N = Gx.shape[0]

# Compute the state similarity matrix. See the paper
print("computing Gs")
Gs = concaust.embed_states(Gx, Gy, normalize=False)

# Compute a spectral basis for representing the causal states. See the paper
print('projection')
eigenvalues, basis, coords = concaust.spectral_basis(Gs, num_basis=10, scaled=False)

# Visualization
import matplotlib.pyplot as plt
foo,bar,plop = plt.hist(coords[:,1],100)
plt.show()

# batch processing
#np.savetxt("even-p%d-f%d-N%d-bw%f-seed%s.txt" % (npast, nfuture, N, bandwidth, seed), right_eigenvectors[:,:3])
#np.savetxt("even-eval-p%d-f%d-N%d-bw%f-seed%s.txt" % (npast, nfuture, N, bandwidth, seed), eigenvalues)

