#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Example code for the paper :
  Discovering Causal Structure with Reproducing-Kernel Hilbert Space ε-Machines
  by Nicolas Brodu and James P. Crutchfield

This code is free software, according to the MIT license
Contact: nicolas.brodu@inria.fr
"""
import numpy as np
import matplotlib.pyplot as plt

def load(sde_noise, added_noise):
    l = np.load('lores/lores_%g_%g.npz' % (sde_noise, added_noise))
    global series
    global valid_series
    global eigval
    global rev
    global dev_d
    global min_d
    global max_d
    global avg_d
    global dev_r
    global min_r
    global max_r
    global avg_r
    series = l['series']
    valid_series = l['valid_series']
    eigval = l['eigval']
    rev = l['rev']
    dev_d = l['dev_d']
    min_d = l['min_d']
    max_d = l['max_d']
    avg_d = l['avg_d']
    dev_r = l['dev_r']
    min_r = l['min_r']
    max_r = l['max_r']
    avg_r = l['avg_r']

# Eigenvalues plot
if False:
    
    plt.figure()
    ax = plt.axes()
    for axis in [ax.xaxis, ax.yaxis]:
        axis.set_major_formatter(FormatStrFormatter('%g'))
    ax.set_xlim((0.75,7.25))
    ax.set_ylim((-0.001,0.0251))
    
    # draw solid lines in the legend
    plt.plot(-10, -10, 'r-')
    plt.plot(-10, -10, 'g-')
    plt.plot(-10, -10, 'b-')
    
    load(1,0)
    plt.plot(np.arange(1,8), eigval[1:8], 'r-', linewidth=2)
    load(0,1)
    plt.plot(np.arange(1,8), eigval[1:8], 'g', linestyle=(0, (10, 10)), linewidth=2)
    load(0,0)
    plt.plot(np.arange(1,8), eigval[1:8], 'b', linestyle=(10, (10, 10)), linewidth=2)
    
    
    plt.legend((r"Eigenvalues for $\eta=1, \nu=0$", r"Eigenvalues for $\eta=0, \nu=1$", r"Eigenvalues for $\eta=0, \nu=0$"), loc='upper right')
    plt.xlabel("Eigenvalue index", fontsize=16)
    plt.ylabel("Eigenvalue", fontsize=16)


# Prediction accuracies
if False:
    plt.figure()
    ax = plt.axes()
    ax.set_yscale("log", nonposy='clip')
    ax.set_xscale("log", nonposx='clip')
    for axis in [ax.xaxis, ax.yaxis]:
        axis.set_major_formatter(FormatStrFormatter('%g'))
    ax.set_xlim((4.5,550))
    ax.set_ylim((0.0002,20))
    
    # outside the display area, for the legend only
    plt.plot(1000,1000, "b-", linewidth=2)
    plt.plot(1000,1000, "r-", linewidth=2)
    plt.plot(1000,1000, "k--", linewidth=2)
    plt.plot(1000,1000, "k:", linewidth=2)
    plt.plot(1000,1000, "k-", linewidth=2)
    
    r = np.arange(1,101) * 5;
    
    load(0,0)
    plt.plot(r, avg_d, "b-", linewidth=2)
    plt.plot(r, avg_r, "r-", linewidth=2)
    
    load(0,1)
    plt.plot(r, avg_d, "b--", linewidth=2)
    plt.plot(r, avg_r, "r--", linewidth=2)
    
    load(1,0)
    plt.plot(r, avg_d, "b:", linewidth=2)
    plt.plot(r, avg_r, "r:", linewidth=2)
    
    plt.legend(("Operator on $(u,v,w)$ triplets", 
                "Operator on RKHS states",
                r"$\eta=0$, $\nu=1$",
                r"$\eta=1$, $\nu=0$",
                r"$\eta=0$, $\nu=0$",
                ), loc='lower right')
    
    plt.xlabel("Prediction steps n $(t = t_0 + n*dt)$", fontsize=16)
    plt.ylabel("Average discrepancy", fontsize=16)


# Sensitivity to added noise for predictions at 50 dt
if False:
    plt.figure()
    ax = plt.axes()
    ax.set_xscale("log")
    ax.set_yscale("log")
    for axis in [ax.xaxis, ax.yaxis]:
        axis.set_major_formatter(FormatStrFormatter('%g'))
        
    ax.set_xlim((0.008,1.2))
    ax.set_ylim((0.1,12.5))
    plt.plot(1000,1000, "b-", linewidth=2)
    plt.plot(1000,1000, "r-", linewidth=2)
    plt.plot(1000,1000, "k--", linewidth=2)
    plt.plot(1000,1000, "k:", linewidth=2)

    index = 9
    noise_levels = (0.01, 0.02, 0.05, 0.1, 0.2, 0.5, 1)
    rmean = np.zeros(len(noise_levels))
    dmean = np.zeros(len(noise_levels))
    for ni, added_noise in enumerate(noise_levels):
        load(0, added_noise)
        dmean[ni] = avg_d[index]
        rmean[ni] = avg_r[index]
    plt.plot(noise_levels, dmean, "b--", marker='o')
    plt.plot(noise_levels, rmean, "r--", marker='o')

    noise_levels = (0.01, 0.02, 0.05, 0.1, 0.2, 0.5, 1)
    rmean = np.zeros(len(noise_levels))
    dmean = np.zeros(len(noise_levels))
    for ni, sde_noise in enumerate(noise_levels):
        load(sde_noise, 0)
        dmean[ni] = avg_d[index]
        rmean[ni] = avg_r[index]
    plt.plot(noise_levels, dmean, "b:", marker='o')
    plt.plot(noise_levels, rmean, "r:", marker='o')

    load(0, 0)
    plt.plot(0.008,avg_d[index], "b", marker="o", zorder=10, clip_on=False)
    plt.plot(0.008,avg_r[index], "r", marker="o", zorder=10, clip_on=False)
    
    plt.legend(("Operator on $(u,v,w)$ triplets", 
                "Operator on RKHS states",
                r"$\eta=0$",
                r"$\nu=0$",
                ), loc='upper left')
    
    plt.xlabel(r"Added noise level ($\eta$ or $\nu$)", fontsize=16)
    plt.ylabel("Average discrepancy at %d prediction steps" % r[index], fontsize=16)


if False:
    import concaust
    
    fig = plt.figure()
    load(0,0)
    ax = fig.add_subplot(321, projection='3d')
    ax.set_axis_off()
    concaust.plot3D(series, ax)
    ax = fig.add_subplot(322, projection='3d')
    ax.set_axis_off()
    concaust.plot3D(rev[:,1:], ax)
    load(0,1)
    ax = fig.add_subplot(323, projection='3d')
    ax.set_axis_off()
    concaust.plot3D(series, ax)
    ax = fig.add_subplot(324, projection='3d')
    ax.set_axis_off()
    concaust.plot3D(rev[:,1:], ax)
    load(1,0)
    ax = fig.add_subplot(325, projection='3d')
    ax.set_axis_off()
    concaust.plot3D(series, ax)
    ax = fig.add_subplot(326, projection='3d')
    ax.set_axis_off()
    concaust.plot3D(rev[:,1:], ax)
    plt.tight_layout()

    